#include <stdint.h>
#include <avr/pgmspace.h>
#include <IRremote.h>

#include "sine_table.h"

//Function prototypes
uint16_t ConvertButtonToNoteSkips(unsigned long value);
void TurnOnLED(void);
void TurnOffLED(void);

#define IR_PIN 2 //Digitial pin for IR reciever
#define repeatCode 0xFFFFFFFF
#define LDR_PIN A0 
#define MAX_ANALOG_IN 1023 // (2^10 - 1) for a 10 bit ADC
#define HIGH_VOLTAGE 5
#define MIN_LDR_VOLTS 2.0
#define MAX_LDR_VOLTS 4.0

float LDR_Voltage = 0;
IRrecv IR_Reciever(IR_PIN); //Instantiate IR reciever object
decode_results results; //Instantiate object to store results
bool buttonPressDetected = false;
uint8_t timeout = 0;
uint16_t baseSkips = 0;
uint16_t adjustSkips = 0;
//uint32_t priorButton = 0;



void setup()
{
  noInterrupts(); //Disable interrupts during setup

  //Initalize digital pins 8-12
  DDRB = DDRB | B00011111;        //Make port B pins 0-4 outputs (i.e. digital pins 8-12)
  PORTB = PORTB & ~(B00011111);   //Set port B pins 0-4 to zero volts

  // initialize timer1
  TCCR1A = 0; //clear settings
  TCCR1B = 0; //clear settings
  TCNT1  = 0; // clear timer count
  //# of ticks = (desired interrupt period)/(tick length) = (1/32000)/(1/16e6) = 500
  OCR1A = 500;              // Timer will match this value every 500 ticks then reset to zero and trigger an interrupt
  TCCR1B |= (1 << WGM12);   // Set timer to CTC (clear timer on compare) mode
  TCCR1B |= B001;           // No prescaler, timer will tike at a rate equal to the clock rate of 16 MHz
  TIMSK1 |= (1 << OCIE1A);  // Set the timer to trigger an interrupt on compare match

  IR_Reciever.enableIRIn(); // Start the IR receiver
  pinMode(13, OUTPUT); //Pin connected to onboard blue LED

  interrupts();             // enable all interrupts
}

void loop()
{
  //Static variables are only calculated the first time through the loop
  //The const keyword indicates these variables will not change while the program is running
  static const uint16_t LDR_Lower_Threshold = MAX_ANALOG_IN * (MIN_LDR_VOLTS+0.25) / HIGH_VOLTAGE;
  static const uint16_t LDR_Upper_Threshold = MAX_ANALOG_IN * MAX_LDR_VOLTS / HIGH_VOLTAGE;
  static const uint8_t MaxAdjustSkips = 0;
  
  
  //Pass the memeory address of the results object to the decode() method
  //The decode() method returns true if any 32 bit signal was detected
  buttonPressDetected = IR_Reciever.decode(&results);
  uint16_t analogValue = constrain(analogRead(LDR_PIN),LDR_Lower_Threshold,LDR_Upper_Threshold);
  adjustSkips = (uint16_t)map(analogValue,LDR_Lower_Threshold,LDR_Upper_Threshold, 0, MaxAdjustSkips);

  if (buttonPressDetected && results.value == repeatCode) {
    timeout = 0;
    TurnOnLED();
    IR_Reciever.resume();
  }

  else if (buttonPressDetected)
  {
    timeout = 0;
    baseSkips = ConvertButtonToNoteSkips(results.value);
    TurnOnLED();
    IR_Reciever.resume();
  }

  else if (timeout > 100)
  {
    baseSkips = 0; //turn off note
    TurnOffLED();
  }
  else
  {
    timeout++;
  }
  delay(1);
}

//This fuction is invoked 32,000 times a second to update the output to the DAC
//The code in loop() never calls this functino, it is instead triggered by timer 1
//ISR stands for Interrupt Service Routine
ISR (TIMER1_COMPA_vect)
{
  static uint16_t counter = 0;
  if (baseSkips == 0)
  { //Set output to zero volts
    PORTB = PORTB & B11100000;
  }
  else
  { //Set output to next value, pgm_read_byte is needed since the table is stored in flash
    // Here '&' is the "address of" operater. &sineTable32[i] is the address of the ith entry
    PORTB = (PORTB & B11100000) | pgm_read_byte(&sineTable32[counter]);
    counter = (counter + baseSkips + adjustSkips) % 4000;
  }

}

uint16_t ConvertButtonToNoteSkips(unsigned long value) {
  uint16_t returnValue = 0;

  //DEFINE BUTTON CODES
  static const uint32_t buttonPower = 0x10EFD827;
  static const uint32_t buttonA = 0x10EFF807;
  static const uint32_t buttonB = 0x10EF7887;
  static const uint32_t buttonC = 0x10EF58A7;
  static const uint32_t buttonUp = 0x10EFA05F;
  static const uint32_t buttonLeft = 0x10EF10EF;
  static const uint32_t buttonCenter = 0x10EF20DF;
  static const uint32_t buttonRight = 0x10EF807F;
  static const uint32_t buttonDown = 0x10EF00FF;

  //DEFINE NOTE FREQUENCIES
  static const uint16_t B4_Hz = 494;

  static const uint16_t C5_Hz = 523;
  static const uint16_t D5_Hz = 587;
  static const uint16_t E5_Hz = 659;
  static const uint16_t F5_Hz = 698;
  static const uint16_t G5_Hz = 784;
  static const uint16_t A5_Hz = 880;
  static const uint16_t B5_Hz = 988;

  //CALCUALTE SKIPS FOR VARIOUS NOTES
  // Skips = (sine table entries / interrupt rate) * frequency = (4000/32000)*f = f/8
  static const uint8_t HzPerSkip = 8;

  static const uint16_t skipsB4 = B4_Hz / HzPerSkip;

  static const uint16_t skipsC5 = C5_Hz / HzPerSkip;
  static const uint16_t skipsD5 = D5_Hz / HzPerSkip;
  static const uint16_t skipsE5 = E5_Hz / HzPerSkip;
  static const uint16_t skipsF5 = F5_Hz / HzPerSkip;
  static const uint16_t skipsG5 = G5_Hz / HzPerSkip;
  static const uint16_t skipsA5 = A5_Hz / HzPerSkip;
  static const uint16_t skipsB5 = B5_Hz / HzPerSkip;

  switch (value) {
    
    case buttonPower :
      returnValue = skipsB4;
      break;
      
    case buttonA :
      returnValue = skipsC5;
      break;
      
    case buttonB :
      returnValue = skipsD5;
      break;
      
    case buttonC :
      returnValue = skipsE5;
      break;
      
    case buttonLeft :
      returnValue = skipsF5;
      break;
      
    case buttonCenter :
      returnValue = skipsG5;
      break;
      
    case buttonRight :
      returnValue = skipsA5;
      break;

      case buttonDown :
      returnValue = skipsB5;
      break;
  }
  return returnValue;

}

void TurnOnLED(void){
  digitalWrite(13, HIGH);
}

void TurnOffLED(void){
  digitalWrite(13, LOW);
}

